@@ -1,7 +1,7 @@
 #
 # Kiwi: a Framework and Enhanced Widgets for Python
 #
-# Copyright (C) 2001-2006 Async Open Source
+# Copyright (C) 2001-2007 Async Open Source
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -69,6 +69,9 @@ class Column(PropertyObject, gobject.GObject):
           the attribute
       - B{data-type}: object I{str}
         - the type of the attribute that will be inserted into the column.
+          Supported data types: bool, int, float, str, unicode,
+          decimal.Decimal, datetime.date, datetime.time, datetime.datetime,
+          gtk.gdk.Pixbuf, L{kiwi.currency.currency}, L{kiwi.python.enum}.
       - B{visible}: bool I{True}
         - specifying if it is initially hidden or shown.
       - B{justify}: gtk.Justification I{None}
@@ -122,7 +125,11 @@ class Column(PropertyObject, gobject.GObject):
           cell is editable or not.
       - B{use_markup}: bool I{False}
         - If true, the text will be rendered with markup
+      - B{expander}: bool I{False}
+        - If True, this column will be used as the tree expander column
     """
+    __gtype_name__ = 'Column'
+    gproperty('attribute', str, flags=(gobject.PARAM_READWRITE | gobject.PARAM_CONSTRUCT_ONLY))
     gproperty('title', str)
     gproperty('data-type', object)
     gproperty('visible', bool, default=True)
@@ -142,6 +149,7 @@ class Column(PropertyObject, gobject.GObject):
     gproperty('use-markup', bool, default=False)
     gproperty('icon-size', gtk.IconSize, default=gtk.ICON_SIZE_MENU)
     gproperty('editable-attribute', str)
+    gproperty('expander', bool, False)
     #gproperty('title_pixmap', str)
 
     # This can be set in subclasses, to be able to allow custom
@@ -156,7 +164,7 @@ class Column(PropertyObject, gobject.GObject):
     # initial properties
     on_attach_renderer = None
 
-    def __init__(self, attribute, title=None, data_type=None, **kwargs):
+    def __init__(self, attribute='', title=None, data_type=None, **kwargs):
         """
         Creates a new Column, which describes how a column in a
         ObjectList should be rendered.
@@ -180,10 +188,10 @@ class Column(PropertyObject, gobject.GObject):
                    " not find the value in the instances: %s" % attribute)
             raise AttributeError(msg)
 
-        self.attribute = attribute
         self.compare = None
         self.from_string = None
 
+        kwargs['attribute'] = attribute
         kwargs['title'] = title or attribute.capitalize()
         if not data_type:
             data_type = str
@@ -215,7 +223,7 @@ class Column(PropertyObject, gobject.GObject):
             kwargs['editable'] = True
 
         PropertyObject.__init__(self, **kwargs)
-        gobject.GObject.__init__(self)
+        gobject.GObject.__init__(self, attribute=attribute)
 
     # This is meant to be subclassable, we're using kgetattr, as
     # a staticmethod as an optimization, so we can avoid a function call.
@@ -230,13 +238,13 @@ class Column(PropertyObject, gobject.GObject):
 
     def __repr__(self):
         namespace = self.__dict__.copy()
-        attr = namespace['attribute']
-        del namespace['attribute']
-        return "<%s %s: %s>" % (self.__class__.__name__, attr, namespace)
+        return "<%s: %s>" % (self.__class__.__name__, namespace)
 
     def as_string(self, data):
         data_type = self.data_type
-        if (self.format or
+        if data is None:
+            text = ''
+        elif (self.format or
             data_type == float or
             data_type == Decimal or
             data_type == currency or
@@ -477,7 +485,10 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
     gsignal('double-click', object)
 
     # row right-clicked
-    gsignal('right-click', object)
+    gsignal('right-click', object, gtk.gdk.Event)
+
+    # row middle-clicked
+    gsignal('middle-click', object, gtk.gdk.Event)
 
     # edited object, attribute name
     gsignal('cell-edited', object, str)
@@ -488,7 +499,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
     gproperty('selection-mode', gtk.SelectionMode,
               default=gtk.SELECTION_BROWSE, nick="SelectionMode")
 
-    def __init__(self, columns=[],
+    def __init__(self, columns=None,
                  objects=None,
                  mode=gtk.SELECTION_BROWSE,
                  sortable=False,
@@ -500,6 +511,8 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         @param sortable:      whether the user can sort the list
         @param model:         gtk.TreeModel to use or None to create one
         """
+        if columns is None:
+            columns = []
         # allow to specify only one column
         if isinstance(columns, Column):
             columns = [columns]
@@ -529,6 +542,11 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         # menu
         self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
         self.set_shadow_type(gtk.SHADOW_ETCHED_IN)
+        # This is required for gobject.new to work, since scrolledwindow.add
+        # requires valid adjustments and they are for some reason not
+        # properly set when using gobject.new.
+        self.set_hadjustment(gtk.Adjustment())
+        self.set_vadjustment(gtk.Adjustment())
 
         if not model:
             model = gtk.ListStore(object)
@@ -593,10 +611,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     def __contains__(self, instance):
         "item in list"
-        for row in self._model:
-            if row[COL_MODEL] == instance:
-                return True
-        return False
+        return bool(self._iters.get(id(instance), False))
 
     def __iter__(self):
         "for item in list"
@@ -632,7 +647,15 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
     def __setitem__(self, arg, item):
         "list[n] = m"
         if isinstance(arg, (int, gtk.TreeIter, str)):
-            self._model[arg] = (item,)
+            model = self._model
+            olditem = model[arg][COL_MODEL]
+            model[arg] = (item,)
+
+            # Update iterator cache
+            iters = self._iters
+            iters[id(item)] = model[arg].iter
+            del iters[id(olditem)]
+
         elif isinstance(arg, slice):
             raise NotImplementedError("slices for list are not implemented")
         else:
@@ -677,9 +700,23 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
                 count += 1
         return count
 
-    def insert(self, index, item):
-        "L.insert(index, item) -- insert object before index"
-        raise NotImplementedError
+    def insert(self, index, instance, select=False):
+        """Inserts an instance to the list
+        @param index: position to insert the instance at
+        @param instance: the instance to be added (according to the columns spec)
+        @param select: whether or not the new item should appear selected.
+        """
+        self._treeview.freeze_notify()
+
+        row_iter = self._model.insert(index, (instance,))
+        self._iters[id(instance)] = row_iter
+
+        if self._autosize:
+            self._treeview.columns_autosize()
+
+        if select:
+            self._select_and_focus_row(row_iter)
+        self._treeview.thaw_notify()
 
     def pop(self, index):
         """
@@ -697,6 +734,23 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         cmp(x, y) -> -1, 0, 1"""
         raise NotImplementedError
 
+    def sort_by_attribute(self, attribute, order=gtk.SORT_ASCENDING):
+        """
+        Sort by an attribute in the object model.
+
+        @param attribute: attribute to sort on
+        @type attribute: string
+        @param order: one of gtk.SORT_ASCENDING, gtk.SORT_DESCENDING
+        @type order: gtk.SortType
+        """
+        def _sort_func(model, iter1, iter2):
+            return cmp(
+                getattr(model[iter1][0], attribute, None),
+                getattr(model[iter2][0], attribute, None))
+        unused_sort_col_id = len(self._columns)
+        self._model.set_sort_func(unused_sort_col_id, _sort_func)
+        self._model.set_sort_column_id(unused_sort_col_id, order)
+
     # Properties
 
     def prop_set_selection_mode(self, mode):
@@ -781,12 +835,6 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         sorted = None
         expand = False
         for column in columns:
-            if column.searchable:
-                if searchable:
-                    raise ValueError("Can't make column %s searchable, column"
-                                     " %s is already set as searchable" % (
-                        column.attribute, searchable.attribute))
-                searchable = column.searchable
             if column.sorted:
                 if sorted:
                     raise ValueError("Can't make column %s sorted, column"
@@ -821,6 +869,9 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
                                       (column, column.attribute))
             treeview_column.set_sort_column_id(index)
 
+        if column.sorted:
+            self._model.set_sort_column_id(index, column.order)
+
         renderer, renderer_prop = self._guess_renderer_for_type(column)
         if column.on_attach_renderer:
             column.on_attach_renderer(renderer)
@@ -884,6 +935,9 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
             if not issubclass(column.data_type, bool):
                 raise TypeError("You can only use radio for boolean columns")
 
+        if column.expander:
+            self._treeview.set_expander_column(treeview_column)
+
         # typelist here may be none. It's okay; justify_columns will try
         # and use the specified justifications and if not present will
         # not touch the column. When typelist is not set,
@@ -1033,10 +1087,10 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     # TreeView
     def _treeview_search_equal_func(self, model, tree_column, key, treeiter, column):
-        "for searching inside the treeview"
+        "for searching inside the treeview, case-insensitive by default"
         data = column.get_attribute(model[treeiter][COL_MODEL],
                                     column.attribute, None)
-        if data.startswith(key):
+        if data.lower().startswith(key.lower()):
             return False
         return True
 
@@ -1057,26 +1111,38 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         item = row[COL_MODEL]
         self.emit('row-activated', item)
 
-    def _on_treeview__button_press_event(self, treeview, event):
-        "Generic button-press-event handler to be able to catch double clicks"
-
+    def _get_selection_or_selected_rows(self):
         selection = self._treeview.get_selection()
         mode = selection.get_mode()
         if mode == gtk.SELECTION_MULTIPLE:
             item = self.get_selected_rows()
         else:
             item = self.get_selected()
+        return item
 
-        if not item:
-            return
+    def _emit_button_press_signal(self, signal_name, event):
+        item = self._get_selection_or_selected_rows()
+        if item:
+            self.emit(signal_name, item, event)
 
+    def _on_treeview__button_press_event(self, treeview, event):
+        "Generic button-press-event handler to be able to catch double clicks"
+
+        # Right and Middle click
+        if event.type == gtk.gdk.BUTTON_PRESS:
+            if event.button == 3:
+                signal_name = 'right-click'
+            elif event.button == 2:
+                signal_name = 'middle-click'
+            else:
+                return
+            gobject.idle_add(self._emit_button_press_signal, signal_name,
+                             event.copy())
         # Double left click
-        if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
-            self.emit('double-click', item)
-
-        # Right click
-        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
-            self.emit('right-click', item)
+        elif event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
+            item = self._get_selection_or_selected_rows()
+            if item:
+                self.emit('double-click', item)
 
     # CellRenderers
     def _cell_data_text_func(self, tree_column, renderer, model, treeiter,
@@ -1120,8 +1186,9 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         row = model[treeiter]
         data = column.get_attribute(row[COL_MODEL],
                                     column.attribute, None)
-        pixbuf = self.render_icon(data, column.icon_size)
-        renderer.set_property(renderer_prop, pixbuf)
+        if data is not None:
+            pixbuf = self.render_icon(data, column.icon_size)
+            renderer.set_property(renderer_prop, pixbuf)
 
     def _cell_data_combo_func(self, tree_column, renderer, model, treeiter,
                               (column, renderer_prop)):
@@ -1157,11 +1224,8 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
                 if value == True:
                     old = obj
                     break
-            else:
-                raise TypeError("You need an initial attribute value set "
-                                "to true when using radio")
-
-        setattr(old, attr, False)
+        if old:
+            setattr(old, attr, False)
 
         # Active new and save a reference to the object of the
         # previously selected row
@@ -1287,6 +1351,12 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         tree_columns = self._treeview.get_columns()
         return tree_columns[index]
 
+    def grab_focus(self):
+        """
+        Grabs the focus of the ObjectList
+        """
+        self._treeview.grab_focus()
+
     def _clear_columns(self):
         # Reset the sort function for all model columns
         model = self._model
@@ -1316,8 +1386,8 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     def append(self, instance, select=False):
         """Adds an instance to the list.
-        - instance: the instance to be added (according to the columns spec)
-        - select: whether or not the new item should appear selected.
+        @param instance: the instance to be added (according to the columns spec)
+        @param select: whether or not the new item should appear selected.
         """
 
         # Freeze and save original selection mode to avoid blinking
@@ -1621,6 +1691,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 type_register(ObjectList)
 
 class ObjectTree(ObjectList):
+    __gtype_name__ = 'ObjectTree'
     def __init__(self, columns=[], objects=None, mode=gtk.SELECTION_BROWSE,
                  sortable=False, model=None):
         if not model:
@@ -1686,10 +1757,10 @@ class ObjectTree(ObjectList):
         if not objid in self._iters:
             raise ValueError("instance %r is not in the list" % instance)
         treeiter = self._iters[objid]
-        
+
         self.get_treeview().expand_row(
             self._model[treeiter].path, open_all)
-        
+
     def collapse(self, instance):
         """
         This method collapses the row specified by path
@@ -1836,7 +1907,7 @@ class SummaryLabel(ListLabel):
         attr = column.attribute
         get_attribute = column.get_attribute
 
-        value = sum([get_attribute(obj, attr) for obj in self._klist],
+        value = sum([get_attribute(obj, attr, 0) for obj in self._klist],
                     column.data_type('0'))
 
         self.set_value(column.as_string(value))
